<?php
/**
 * Cryptocoin Payments Admin Controller
 */
class ControllerExtensionPaymentCoinpayment extends Controller {

	/** @var array $error Validation errors */
	private $error = array();

	/** @var boolean $ajax Whether the request was made via AJAX */
	private $ajax = false;

	/** @var CoinPaymentLibrary $coinpayment */
	private $coinpayment;

	/**
	 * Cryptocoin Payments Admin Controller Constructor
	 * @param Registry $registry
	 */
	public function __construct($registry) {
		parent::__construct($registry);

		// Make langauge strings and CoinPayment Library available to all
		$this->load->language('extension/payment/coinpayment');
		//var_dump($registry);die;
		
		//$this->coinpayment = new Coinpayments($registry);

		// Setup logging
		//$this->logger = new Log('coinpayment.log');

		// Is this an ajax request?
		if (!empty($this->request->server['HTTP_X_REQUESTED_WITH']) && strtolower($this->request->server['HTTP_X_REQUESTED_WITH']) == 'xmlhttprequest')
		{
			$this->ajax = true;
			$this->response->addHeader('Content-type: application/json');
		}
	}

	/**
	 * Primary settings page
	 * @return void
	 */
	public function index() {
		$this->load->model('setting/setting');
		
		// Saving settings
		if (($this->request->server['REQUEST_METHOD'] == 'POST') && $this->request->post['action'] === 'save' && $this->validate()) {
			$this->model_setting_setting->editSetting('payment_coinpayment', $this->request->post);
			/*
			$this->config->get('payment_coinpayment_merchant_id', $this->request->post['coinpayment_merchant_id']);
			$this->config->get('payment_coinpayment_gateway', $this->request->post['coinpayment_gateway']);
			$this->config->get('payment_coinpayment_geo_zone_id', $this->request->post['coinpayment_geo_zone_id']);
			$this->config->get('payment_coinpayment_payment_coinpayment_status', $this->request->post['coinpayment_status']);
			$this->config->get('payment_coinpayment_sort_order', $this->request->post['coinpayment_sort_order']);
			$this->config->get('payment_coinpayment_pending_status', $this->request->post['coinpayment_pending_status']);
			$this->config->get('payment_coinpayment_paid_status', $this->request->post['coinpayment_paid_status']);
			$this->config->get('payment_coinpayment_ipn_url', $this->request->post['coinpayment_ipn_url']);
			$this->config->get('payment_coinpayment_success_url', $this->request->post['coinpayment_success_url']);
			$this->config->get('payment_coinpayment_debug', $this->request->post['coinpayment_debug']);
			*/
			$this->session->data['success'] = $this->language->get('text_success')." / news: ".$this->request->post['payment_coinpayment_status'];
			$this->response->redirect($this->url->link('extension/payment/coinpayment', 'user_token=' . $this->session->data['user_token'], 'SSL'));
		}

		// Send Support Request form submitted
		//if (($this->request->server['REQUEST_METHOD'] == 'POST') && $this->request->post['action'] === 'send' && $this->validateSupportRequest()) {
			//$this->coinpayment->sendSupportRequest();
			//$this->session->data['success'] = $this->language->get('success_support_request');
		//}

		$this->document->setTitle($this->language->get('heading_title'));

		// #HEADER and globals
		$data['heading_title'] = $this->language->get('heading_title');

		$data['text_general'] = $this->language->get('text_general');
		$data['text_statuses'] = $this->language->get('text_statuses');
		$data['text_advanced'] = $this->language->get('text_advanced');
		$data['text_edit'] = $this->language->get('text_edit');
		$data['text_enabled'] = $this->language->get('text_enabled');
		$data['text_disabled'] = $this->language->get('text_disabled');
		$data['text_all_zones'] = $this->language->get('text_all_zones');
		$data['text_yes'] = $this->language->get('text_yes');
		$data['text_no'] = $this->language->get('text_no');
		$data['text_are_you_sure'] = $this->language->get('text_are_you_sure');

		$data['entry_geo_zone'] = $this->language->get('entry_geo_zone');
		$data['entry_status'] = $this->language->get('entry_status');
		$data['entry_sort_order'] = $this->language->get('entry_sort_order');
		$data['entry_paid_status'] = $this->language->get('entry_paid_status');
		$data['entry_confirmed_status'] = $this->language->get('entry_confirmed_status');
		$data['entry_complete_status'] = $this->language->get('entry_complete_status');
		$data['entry_ipn_url'] = $this->language->get('entry_ipn_url');
		$data['entry_success_url'] = $this->language->get('entry_success_url');
		$data['entry_debug'] = $this->language->get('entry_debug');

		$data['help_api_access'] = $this->language->get('help_api_access');
		$data['help_paid_status'] = $this->language->get('help_paid_status');
		$data['help_confirmed_status'] = $this->language->get('help_confirmed_status');
		$data['help_complete_status'] = $this->language->get('help_complete_status');
		$data['help_ipn_url'] = $this->language->get('help_ipn_url');
		$data['help_success_url'] = $this->language->get('help_success_url');
		$data['help_debug'] = $this->language->get('help_debug');

		$data['button_save'] = $this->language->get('button_save');
		$data['button_cancel'] = $this->language->get('button_cancel');
		$data['button_clear'] = $this->language->get('button_clear');
		$data['button_continue'] = $this->language->get('button_continue');

		$data['tab_settings'] = $this->language->get('tab_settings');

		$data['url_action'] = $this->url->link('extension/payment/coinpayment', 'user_token=' . $this->session->data['user_token'], 'SSL');
		$data['url_cancel'] = $this->url->link('extension/extension', 'user_token=' . $this->session->data['user_token'], 'SSL');
		$data['url_reset'] = $this->url->link('extension/payment/coinpayment/reset', 'user_token=' . $this->session->data['user_token'], 'SSL');

		$data['breadcrumbs'] = array();
		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_home'),
			'href' => $this->url->link('common/dashboard', 'user_token=' . $this->session->data['user_token'], 'SSL')
		);
		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_payment'),
			'href' => $this->url->link('marketplace/extension', 'user_token=' . $this->session->data['user_token'].'&type=payment', 'SSL')
		);
		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('heading_title'),
			'href' => $this->url->link('extension/payment/coinpayment', 'user_token=' . $this->session->data['user_token'], 'SSL')
		);

		// #GENERAL
		$this->load->model('localisation/geo_zone');
		$data['geo_zones'] = $this->model_localisation_geo_zone->getGeoZones();

		$data['payment_coinpayment_merchant_id'] = (isset($this->request->post['payment_coinpayment_merchant_id'])) ? $this->request->post['payment_coinpayment_merchant_id'] : $this->config->get('payment_coinpayment_merchant_id');
		$data['payment_coinpayment_gateway'] = (isset($this->request->post['payment_coinpayment_gateway'])) ? $this->request->post['payment_coinpayment_gateway'] : $this->config->get('payment_coinpayment_gateway');
		$data['payment_coinpayment_geo_zone_id'] = (isset($this->request->post['payment_coinpayment_geo_zone_id'])) ? $this->request->post['payment_coinpayment_geo_zone_id'] : $this->config->get('payment_coinpayment_geo_zone_id');
		$data['payment_coinpayment_status'] = (isset($this->request->post['payment_coinpayment_status'])) ? $this->request->post['payment_coinpayment_status'] : $this->config->get('payment_coinpayment_status');
		$data['payment_coinpayment_sort_order'] = (isset($this->request->post['payment_coinpayment_sort_order'])) ? $this->request->post['payment_coinpayment_sort_order'] : $this->config->get('payment_coinpayment_sort_order');

		// #ORDER STATUSES
		$this->load->model('localisation/order_status');
		$data['order_statuses'] = $this->model_localisation_order_status->getOrderStatuses();
		$data['payment_coinpayment_paid_status'] = (isset($this->request->post['payment_coinpayment_paid_status'])) ? $this->request->post['payment_coinpayment_paid_status'] : $this->config->get('payment_coinpayment_paid_status');
		$data['payment_coinpayment_pending_status'] = (isset($this->request->post['payment_coinpayment_pending_status'])) ? $this->request->post['payment_coinpayment_pending_status'] : $this->config->get('payment_coinpayment_pending_status');

		// #ADVANCED
		$data['payment_coinpayment_ipn_url'] = (isset($this->request->post['payment_coinpayment_ipn_url'])) ? $this->request->post['payment_coinpayment_ipn_url'] : $this->config->get('payment_coinpayment_ipn_url');
		$data['payment_coinpayment_success_url'] = (isset($this->request->post['payment_coinpayment_success_url'])) ? $this->request->post['payment_coinpayment_success_url'] : $this->config->get('payment_coinpayment_success_url');

		if($this->config->get('config_secure')) {
			$default_ipn_url = $this->url->link('extension/payment/coinpayment/callback', $this->config->get('config_secure'));
			$default_success_url = $this->url->link('extension/payment/coinpayment/success', $this->config->get('config_secure'));
		}else{
			$default_ipn_url = $this->url->link('extension/payment/coinpayment/callback');
			$default_success_url = $this->url->link('extension/payment/coinpayment/success');
		}
 		$default_ipn_url = str_replace(HTTP_SERVER, HTTP_CATALOG, $default_ipn_url);
		$default_ipn_url = str_replace(HTTPS_SERVER, HTTPS_CATALOG, $default_ipn_url);
		$data['default_ipn_url'] = $default_ipn_url;
		if (empty($data['payment_coinpayment_ipn_url'])) {
			$data['payment_coinpayment_ipn_url'] = $default_ipn_url;
			$this->config->set('ipn_url', $default_ipn_url);			
		}

		$default_success_url = str_replace(HTTP_SERVER, HTTP_CATALOG, $default_success_url);
		$default_success_url = str_replace(HTTPS_SERVER, HTTPS_CATALOG, $default_success_url);
		$data['default_success_url'] = $default_success_url;
		if (empty($data['payment_coinpayment_success_url'])) {
			$data['payment_coinpayment_success_url'] = $default_success_url;
			$this->config->set('success_url', $default_success_url);
		}

		$data['payment_coinpayment_debug'] = (isset($this->request->post['payment_coinpayment_debug'])) ? $this->request->post['payment_coinpayment_debug'] : $this->config->get('payment_coinpayment_debug');

		// #LOG
		$file = DIR_LOGS . 'coinpayment.log';
		$data['log'] = '';

		$matches_array = array();

		if (file_exists($file)) {
			$lines = file($file, FILE_USE_INCLUDE_PATH, null);
			foreach ($lines as $line_num => $line) {
				if (preg_match('/^([^\\[]*)\\[([^\\]]*)\\](\\{([^}]*)\\})?(.*)/', $line, $matches)) {
					unset($matches[3]);
					$level = strtolower($matches[2]);
					$matches[0] = '';
					$matches[1] = '<span class="bp-log-date">' . $matches[1] . '</span>';
					$matches[2] = '<span class="bp-log-level">[<span>' . $matches[2] . '</span>]</span>';
					if (!empty($matches[4])) {
						$matches[4] = '<span class="bp-log-locale">{<span>' . $matches[4] . '</span>}</span>';
						$matches[4] = preg_replace('/((->)|(::))/', '<span>$1</span>', $matches[4]);
					}
					$matches[5] = '<span class="bp-log-message">' . $matches[5] . '</span>';
					$line = '<span class="bp-log bp-log-' . $level . '">' . implode('', $matches) . '</span>';
				}

				$data['log'] .= '<div>' . $line . "</div>\n";
			}
		}

		$data['url_clear'] = $this->url->link('extension/payment/coinpayment/clear', 'user_token=' . $this->session->data['user_token'], 'SSL');

		// #LAYOUT
		$data['header'] = $this->load->controller('common/header');
		$data['column_left'] = $this->load->controller('common/column_left');
		$data['footer'] = $this->load->controller('common/footer');

		// #NOTIFICATIONS
		$data['error_warning'] = '';
		if (isset($this->error['warning'])) {
			$data['error_warning'] = $this->error['warning'];
		} elseif (isset($this->session->data['warning'])) {
			$data['error_warning'] = $this->session->data['warning'];
			unset($this->session->data['warning']);
		} else {
			$data['error_warning'] = '';
		}

		$data['success'] = '';
		if (isset($this->session->data['success'])) {
			$data['success'] = $this->session->data['success'];
			unset($this->session->data['success']);
		}

		$data['error_status'] = '';
		if (isset($this->error['status'])) {
			$data['error_status'] = $this->error['status'];
		}

		$data['error_ipn_url'] = '';
		if (isset($this->error['ipn_url'])) {
			$data['error_ipn_url'] = $this->error['ipn_url'];
		}

		$data['error_success_url'] = '';
		if (isset($this->error['success_url'])) {
			$data['error_success_url'] = $this->error['success_url'];
		}

		$this->response->setOutput($this->load->view('extension/payment/coinpayment', $data));
	}

	/**
	 * Clears the CoinPayments log
	 * @return void
	 */
	public function clear() {
		$file = DIR_LOGS . 'coinpayment.log';
		$handle = fopen($file, 'w+');
		fclose($handle);

		$this->session->data['success'] = $this->language->get('success_clear');
		$this->response->redirect($this->url->link('extension/payment/coinpayment', 'user_token=' . $this->session->data['user_token'], 'SSL'));
	}

	/**
	 * Convenience wrapper for coinpayment logs
	 * @param string $level The type of log.
	 *					  Should be 'error', 'warn', 'info', 'debug', 'trace'
	 *					  In normal mode, 'error' and 'warn' are logged
	 *					  In debug mode, all are logged
	 * @param string $message The message of the log
	 * @param int $depth Depth addition for debug backtracing
	 * @return void
	 */
	private function log($level, $message, $depth = 0) {
		$depth += 1;
		//$this->coinpayment->log($level, $message, $depth);
	}

	/**
	 * Validate the primary settings for the CoinPayments extension
	 * @return boolean True if the settings provided are valid
	 */
	private function validate() {
		if (!$this->user->hasPermission('modify', 'extension/payment/coinpayment')) {
			$this->error['warning'] = $this->language->get('warning_permission');
		}
		if (!$this->request->post['payment_coinpayment_merchant_id']) {
			$this->error['merchant_id'] = $this->language->get('error_merchant_id');
		}
		if (!$this->request->post['payment_coinpayment_ipn_url'] || false === filter_var($this->request->post['payment_coinpayment_ipn_url'], FILTER_VALIDATE_URL)) {
			$this->error['ipn_url'] = $this->language->get('error_ipn_url');
		}
		if (!$this->request->post['payment_coinpayment_success_url'] || false === filter_var($this->request->post['payment_coinpayment_success_url'], FILTER_VALIDATE_URL)) {
			$this->error['success_url'] = $this->language->get('error_success_url');
		}

		return !$this->error;
	}

	/**
	 * Install the extension by setting up some smart defaults
	 * @return void
	 */
	public function install() {
		$this->load->model('localisation/order_status');
		$order_statuses = $this->model_localisation_order_status->getOrderStatuses();
		$default_pending = null;
		$default_paid = null;
		foreach ($order_statuses as $order_status) {
			if ($order_status['name'] == 'Processing') {
				$default_pending = $order_status['order_status_id'];
			} elseif ($order_status['name'] == 'Complete') {
				$default_paid = $order_status['order_status_id'];
			}
		}

		$this->load->model('setting/setting');
		$default_settings = array(
			'payment_coinpayment_merchant_id' => '',
			'payment_coinpayment_gateway' => '',
			'payment_coinpayment_geo_zone_id' => '0',
//			'payment_coinpayment_status' => '0',
			'payment_coinpayment_sort_order' => null,
			'payment_coinpayment_paid_status' => $default_paid,
			'payment_coinpayment_pending_status' => $default_pending,
			'payment_coinpayment_ipn_url' => null,
			'payment_coinpayment_success_url' => null,
			'payment_coinpayment_debug' => '0'
			//'payment_coinpayment_version' => $this->coinpayment->version,
		);
		$this->model_setting_setting->editSetting('coinpayment', $default_settings);
	}

	/**
	 * Uninstall the extension by removing the settings
	 * @return void
	 */
	public function uninstall() {
		$this->load->model('setting/setting');
		$this->model_setting_setting->deleteSetting('coinpayment');
	}
}
