<?php
/**
 * Cryptocoin Payments Controller
 */
class ControllerExtensionPaymentCoinpayment extends Controller {

	/** @var boolean $ajax Whether the request was made via AJAX */
	private $ajax = false;

	/** @var CoinPaymentLibrary $coinpayment */
	private $coinpayment;

	/**
	 * Cryptocoin Payments Controller Constructor
	 * @param Registry $registry
	 */
	public function __construct($registry) {
		parent::__construct($registry);

		// Make langauge strings and Cryptocoin Payments Library available to all
		$this->load->language('extension/payment/coinpayment');
		//$this->coinpayment = new Coinpayment($registry);

		// Setup logging
		//$this->logger = new Log('coinpayment.log');

		// Is this an ajax request?
		if (!empty($this->request->server['HTTP_X_REQUESTED_WITH']) &&
			strtolower($this->request->server['HTTP_X_REQUESTED_WITH']) == 'xmlhttprequest')
		{
			$this->ajax = true;
		}
	}

	/**
	 * Displays the Payment Method (a redirect button)
	 * @return void
	 */
	public function index() {
		//$data['url_redirect'] = $this->url->link('extension/payment/coinpayment/confirm', $this->config->get('config_secure'));
		$data['button_confirm'] = $this->language->get('button_confirm');

		if (isset($this->session->data['error_coinpayment'])) {
			unset($this->session->data['error_coinpayment']);
		}

		$data['action'] = $this->config->get('payment_coinpayment_gateway');

		$this->load->model('checkout/order');
		if (!isset($this->session->data['order_id'])) {
			$this->response->redirect($this->url->link('checkout/cart'));
			return;
		}
		$order_info = $this->model_checkout_order->getOrder($this->session->data['order_id']);


		if ($order_info) {
			$data['merchant'] = $this->config->get('payment_coinpayment_merchant_id');
			$data['item_name'] = html_entity_decode($this->config->get('config_name'), ENT_QUOTES, 'UTF-8');

			//there doesn't seem to be an easy way to separate the shipping and tax into individual values, so we don't bother.
			//if we combined them into one of the tax or shipping fields people might get confused and say the shipping cost is wrong
			$total = $this->currency->format($order_info['total'], $order_info['currency_code'], false, false);

			//Get Produts
			$product_data = array();
			foreach ($this->session->data['cart_paypal']['cart_products'] as $product) {
				$option_data = array();
				foreach ($product['option'] as $option) {
					if ($option['type'] != 'file') {	
						$option_data[] = array(
							'product_option_id'       => $option['product_option_id'],
							'product_option_value_id' => $option['product_option_value_id'],
							'product_option_id'       => $option['product_option_id'],
							'product_option_value_id' => $option['product_option_value_id'],
							'option_id'               => $option['option_id'],
							'option_value_id'         => $option['option_value_id'],								   
							'name'                    => $option['name'],
							'value'                   => $option['option_value'],
							'type'                    => $option['type']
						);					
					} else {
						$this->load->library('encryption');
						
						$encryption = new Encryption($this->config->get('config_encryption'));
						
						$option_data[] = array(
							'product_option_id'       => $option['product_option_id'],
							'product_option_value_id' => $option['product_option_value_id'],
							'product_option_id'       => $option['product_option_id'],
							'product_option_value_id' => $option['product_option_value_id'],
							'option_id'               => $option['option_id'],
							'option_value_id'         => $option['option_value_id'],								   
							'name'                    => $option['name'],
							'value'                   => $encryption->decrypt($option['option_value']),
							'type'                    => $option['type']
						);								
					}
				}
				
				$product_data[] = array(
					'product_id' => $product['product_id'],
					'name'       => $product['name'],
					'model'      => $product['model'],
					'option'     => $option_data,
					'download'   => $product['download'],
					'quantity'   => $product['quantity'],
					'subtract'   => $product['subtract'],
					'price'      => $product['price'],
					'total'      => $product['total'],
					'tax'        => $this->tax->getTax($product['price'], $product['tax_class_id']),
					'reward'     => $product['reward']
				); 
			}
			$data['amount'] = $total;
			$data['shipping'] = 0.00;
			$data['tax'] = 0.00;
			$data['products'] = $product_data;
			$data['currency_code'] = $order_info['currency_code'];
			$data['first_name'] = html_entity_decode($order_info['shipping_firstname'], ENT_QUOTES, 'UTF-8');	
			$data['last_name'] = html_entity_decode($order_info['shipping_lastname'], ENT_QUOTES, 'UTF-8');	
			$data['address1'] = html_entity_decode($order_info['shipping_address_1'], ENT_QUOTES, 'UTF-8');	
			$data['address2'] = html_entity_decode($order_info['shipping_address_2'], ENT_QUOTES, 'UTF-8');	
			$data['city'] = html_entity_decode($order_info['shipping_city'], ENT_QUOTES, 'UTF-8');	
			$data['state'] = html_entity_decode($order_info['shipping_zone_code'], ENT_QUOTES, 'UTF-8');	
			$data['zip'] = html_entity_decode($order_info['shipping_postcode'], ENT_QUOTES, 'UTF-8');	
			$data['phone'] = html_entity_decode($order_info['shipping_telephone'], ENT_QUOTES, 'UTF-8');				
			$data['country'] = $order_info['shipping_iso_code_2'];
			$data['email'] = $this->session->data['shipping_address']['email'];
			//$data['invoice'] = $this->session->data['order_id'] . ' - ' . html_entity_decode($order_info['payment_firstname'], ENT_QUOTES, 'UTF-8') . ' ' . html_entity_decode($order_info['payment_lastname'], ENT_QUOTES, 'UTF-8');
			$data['invoice'] = $this->session->data['order_id'];
			$data['success_url'] = $this->config->get('payment_coinpayment_success_url');//$this->url->link('checkout/success');
			$data['ipn_url'] = $this->config->get('payment_coinpayment_ipn_url');//$this->url->link('payment/coinpayment/callback', '', 'SSL');
			$data['cancel_url'] = $this->url->link('checkout/checkout', '', 'SSL');
			
			$data['custom'] = $this->session->data['order_id'];
		}

		if (file_exists(DIR_TEMPLATE . $this->config->get('config_template') . '/template/extension/payment/coinpayment')) {
			return $this->load->view($this->config->get('config_template') . '/template/extension/payment/coinpayment', $data);
		} else {
			return $this->load->view('extension/payment/coinpayment', $data);
		}
	}

	/**
	 * Convenience wrapper for coinpayment logs
	 * @param string $level The type of log.
	 *					  Should be 'error', 'warn', 'info', 'debug', 'trace'
	 *					  In normal mode, 'error' and 'warn' are logged
	 *					  In debug mode, all are logged
	 * @param string $message The message of the log
	 * @param int $depth Depth addition for debug backtracing
	 * @return void
	 */
	public function log($level, $message, $depth = 0) {
		$depth += 1;
		//$this->coinpayment->log($level, $message, $depth);
	}

	/**
	 * Success return page
	 *
	 * Progresses the order if valid, and redirects to OpenCart's Checkout Success page
	 *
	 * @return void
	 */
	public function success() {
		$this->load->model('checkout/order');
		$order_id = $this->session->data['order_id'];
		if (is_null($order_id)) {
			$this->response->redirect($this->url->link('checkout/success'));
			return;
		}

		// Progress the order status
		$this->model_checkout_order->addOrderHistory($order_id, $this->config->get('payment_coinpayment_pending_status'));
		$this->response->redirect($this->url->link('checkout/success'));
	}

	/**
	 * IPN Handler
	 * @return void
	 */
	public function callback() {
		$report = true;
		$error_msg = "";
		
		$auth_ok = false;
		$ipn_mode = isset($_POST['ipn_mode']) ? $_POST['ipn_mode']:'';
		if ($ipn_mode == 'hmac') {
			if (isset($_SERVER['HTTP_HMAC']) && !empty($_SERVER['HTTP_HMAC'])) {
				$request = file_get_contents('php://input');
				if ($request !== FALSE && !empty($request)) {
					$hmac = hash_hmac("sha512", $request, trim($this->config->get('payment_coinpayment_ipn_secret')));
					if ($hmac == $_SERVER['HTTP_HMAC']) {
						$auth_ok = true;
					} else {
						$error_msg = 'HMAC signature does not match';
					}
				} else {
					$error_msg = 'Error reading POST data';
				}
			} else {
				$error_msg = 'No HMAC signature sent.';
			}
		} else {
			$error_msg = 'Unknown IPN Verification Method';
		}
		
		if ($auth_ok) {
			if (isset($this->request->post['invoice'])) {
				$order_id = $this->request->post['invoice'];
			} else {
				$order_id = 0;
			}		
			
			$this->load->model('checkout/order');
					
			$order_info = $this->model_checkout_order->getOrder($order_id);
			
			if ($order_info) {								
				if ($this->request->post['ipn_type'] == "button" || $this->request->post['ipn_type'] == "simple") {
					if ($this->request->post['merchant'] == trim($this->config->get('payment_coinpayment_merchant_id'))) {
						if ($this->request->post['currency1'] == $order_info['currency_code']) {
							if ($this->request->post['amount1'] >= $this->currency->format($order_info['total'], $order_info['currency_code'], false, false)) {
								$report = false;
								$status = (int)$this->request->post['status'];
								$status_text = $this->request->post['status_text'];
								$txn_id = $this->request->post['txn_id'];
								if ($status == 1 || $status == 2 || $status >= 100) {
									$order_status_id = $this->config->get('payment_coinpayment_paid_status');
								} else if ($status >= 0) {
									$order_status_id = $this->config->get('payment_coinpayment_pending_status');
								} else {
									$order_status_id = 0;
								}
								if ($order_status_id) {
									if (!$order_info['order_status_id'] || $order_info['order_status_id'] != $order_status_id) {
										if (!$order_info['order_status_id']) {
											//always confirm/update for the first IPN
											$this->log->write('trace', 'IPN : Confirming order '.$order_id.' with status: '.$order_status_id.' (ipn status code: '.$status.' - '.$status_text.')');
											$this->model_checkout_order->addOrderHistory($order_id, $order_status_id, 'Status: '.$status_text."\nPayment ID: ".$txn_id);
										} else if ($order_info['order_status_id'] != $this->config->get('payment_coinpayment_paid_status')) {
											$this->log->write('trace', 'IPN : Updating order '.$order_id.' with status: '.$order_status_id.' (ipn status code: '.$status.' - '.$status_text.')');
											$this->model_checkout_order->addOrderHistory($order_id, $order_status_id, 'Status: '.$status_text."\nPayment ID: ".$txn_id);
										}
									}
								}
							} else {
								$error_msg = "Amount received is less than the total!";
							}
						} else {
							$error_msg = "Original currency doesn't match!";
						}
					} else {
						$error_msg = "Merchant ID doesn't match!";
					}
				} else {
					$error_msg = "ipn_type != button";
				}
			} else {
				$error_msg = "Could not find order info for order: ".$order_id;
			}
		}
		
		if ($report) {
			$report = "AUTH User: ".$_SERVER['PHP_AUTH_USER']."\n";
			$report .= "AUTH Pass: ".$_SERVER['PHP_AUTH_PW']."\n\n";
			
			if (!empty($error_msg)) {
				$report .= "Error Message: ".$error_msg."\n\n";
			}
			
			$report .= "POST Fields\n\n";
			foreach ($this->request->post as $key => $value) {
				$report .= $key . '=' . html_entity_decode($value, ENT_QUOTES, 'UTF-8'). "\n";
			}
			
			$this->log->write('error', 'IPN : ' . $report);
			//if ($this->config->get('coinpayment_debug_email')) { mail($this->config->get('coinpayment_debug_email'), "Invalid IPN", $report); }
			die('IPN Error: '.$error_msg);
		} else {
			die('IPN OK');
		}
	}
}
